/*

    File: basic bumpmap.vert

    Copyright (C) 2000-2009 Christopher Moore (christopher.e.moore@gmail.com)
	  
    This software is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License along
    with this program; if not, write the Free Software Foundation, Inc., 51
    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/
varying vec3 lightVec; 
varying vec3 eyeVec;
varying vec2 texCoord;

void main(void)
{
	vec3 vTangent = gl_MultiTexCoord1.xyz;
	float handedness = gl_MultiTexCoord1.w;
	
	gl_Position = ftransform();
	texCoord = gl_MultiTexCoord0.xy;
	
	vec3 n = normalize(gl_NormalMatrix * gl_Normal);
	vec3 t = normalize(gl_NormalMatrix * vTangent);
	vec3 b = cross(n, t) * handedness;

	//grab the vertex coordinate in world space
	vec3 vVertex = vec3(gl_ModelViewMatrix * gl_Vertex);
	
	//grab the light vector in world space
	vec3 tmpVec = gl_LightSource[0].position.xyz - vVertex;

	//compute it by the inverse of the [tnb] basis
	lightVec.x = dot(tmpVec, t);
	lightVec.y = dot(tmpVec, b);
	lightVec.z = dot(tmpVec, n);

	//same with the vector from the vertex to the eye
	tmpVec = -vVertex;
	eyeVec.x = dot(tmpVec, t);
	eyeVec.y = dot(tmpVec, b);
	eyeVec.z = dot(tmpVec, n);
}