/*

    File: bumpmap.frag

    Copyright (C) 2000-2009 Christopher Moore (christopher.e.moore@gmail.com)
	  
    This software is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
  
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
  
    You should have received a copy of the GNU General Public License along
    with this program; if not, write the Free Software Foundation, Inc., 51
    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/
varying vec3 lightVec;
varying vec3 eyeVec;
varying vec2 texCoord;
uniform sampler2D colorMap;
uniform sampler2D normalMap;

void main (void) {
	
	float lightDistSqr = dot(lightVec,lightVec);
	float lightDist = sqrt(lightDistSqr);
	
	//3% decrease for using division instead of multiplication
	float attenuation = 1.0 / (gl_LightSource[0].constantAttenuation +
								gl_LightSource[0].linearAttenuation * lightDist +
								gl_LightSource[0].quadraticAttenuation * lightDistSqr);

	vec3 unitLightVec = lightVec / lightDist;

	vec3 unitEyeVec = normalize(eyeVec);
	
	vec4 colorMapTexel = texture2D(colorMap, texCoord);
	
	vec3 bumpNormal = normalize( texture2D(normalMap, texCoord).xyz * 2.0 - 1.0);

	vec4 ambientColor = gl_LightSource[0].ambient * gl_FrontMaterial.ambient;

	//do all-at-once without testing surface dot light vector because
	//its much faster to calc and waste the diffuse & specular
	//than it is to conditionally use them
	
	float diffuseCoeff = max( dot(unitLightVec, bumpNormal), 0.0 );
	vec4 diffuseColor = gl_LightSource[0].diffuse * gl_FrontMaterial.diffuse * diffuseCoeff;

	float specularCoeff = pow(clamp(dot(reflect(-unitEyeVec, bumpNormal), unitLightVec), 0.0, 1.0), gl_FrontMaterial.shininess );
	vec4 specularColor = gl_LightSource[0].specular * gl_FrontMaterial.specular * specularCoeff;
	
	gl_FragColor = ambientColor + (diffuseColor*colorMapTexel + specularColor) * attenuation;
}